@@ -70,49 +70,66 @@ class ChaosAgent:
         """
         Load deployed ERC-8004 v1.0 contract addresses.
         
-        All networks use deterministic deployment addresses for the v1.0 contracts.
-        This ensures consistent addresses across all supported chains.
+        These are the official ERC-8004 v1.0 contracts deployed on testnets.
+        Source: /Users/sumeet/Desktop/erc-8004-contracts/contracts
         """
-        # ERC-8004 v1.0 Deterministic Addresses (same on all networks)
-        ERC8004_V1_IDENTITY = '0x7177a6867296406881E20d6647232314736Dd09A'
-        ERC8004_V1_REPUTATION = '0xB5048e3ef1DA4E04deB6f7d0423D06F63869e322'
-        ERC8004_V1_VALIDATION = '0x662b40A526cb4017d947e71eAF6753BF3eeE66d8'
-        
-        # Network-specific configuration
+        # Network-specific configuration with actual deployed addresses
         contract_addresses = {
             NetworkConfig.BASE_SEPOLIA: {
-                'identity_registry': ERC8004_V1_IDENTITY,
-                'reputation_registry': ERC8004_V1_REPUTATION,
-                'validation_registry': ERC8004_V1_VALIDATION,
+                'identity_registry': '0x8004AA63c570c570eBF15376c0dB199918BFe9Fb',
+                'reputation_registry': '0x8004bd8daB57f14Ed299135749a5CB5c42d341BF',
+                'validation_registry': '0x8004C269D0A5647E51E121FeB226200ECE932d55',
                 'usdc_token': '0x036CbD53842c5426634e7929541eC2318f3dCF7e',
                 'treasury': '0x20E7B2A2c8969725b88Dd3EF3a11Bc3353C83F70'
             },
             NetworkConfig.ETHEREUM_SEPOLIA: {
-                'identity_registry': ERC8004_V1_IDENTITY,
-                'reputation_registry': ERC8004_V1_REPUTATION,
-                'validation_registry': ERC8004_V1_VALIDATION,
+                'identity_registry': '0x8004a6090Cd10A7288092483047B097295Fb8847',
+                'reputation_registry': '0x8004B8FD1A363aa02fDC07635C0c5F94f6Af5B7E',
+                'validation_registry': '0x8004CB39f29c09145F24Ad9dDe2A108C1A2cdfC5',
                 'usdc_token': '0x1c7D4B196Cb0C7B01d743Fbc6116a902379C7238',
-                'treasury': '0x20E7B2A2c8969725b88Dd3EF3a11Bc3353C83F70'
+                'treasury': '0x20E7B2A2c8969725b88Dd3EF3a11Bc3353C83F70',
+                # ChaosChain Protocol Contracts (deployed Nov 16, 2025)
+                'chaos_registry': '0x0D28e47E4b2Bc1a7ca300b88698a9D55112Ec7Cd',
+                'chaos_core': '0x6268C0793891Bc1dD3284Ad8443FAa35a585cf28',
+                'rewards_distributor': '0xA29e2f232CB818fc63691E7C509c5afb082bd5a5',
+                # LogicModules
+                'prediction_logic': '0x32e3086b9Db2667Cd261b195E8fF669C959738C3',
+                'finance_logic': '0xC2B686C4EBA34701d0cC7f250D05B3c62c7CF492',
+                'creative_logic': '0xe6775EdC0A0D9BA7E198F435aEa07D34bC24Fdf2'
             },
             NetworkConfig.OPTIMISM_SEPOLIA: {
-                'identity_registry': ERC8004_V1_IDENTITY,
-                'reputation_registry': ERC8004_V1_REPUTATION,
-                'validation_registry': ERC8004_V1_VALIDATION,
+                'identity_registry': '0x0000000000000000000000000000000000000000',  # Not yet deployed
+                'reputation_registry': '0x0000000000000000000000000000000000000000',
+                'validation_registry': '0x0000000000000000000000000000000000000000',
                 'usdc_token': '0x5fd84259d66Cd46123540766Be93DFE6D43130D7',
                 'treasury': '0x20E7B2A2c8969725b88Dd3EF3a11Bc3353C83F70'
             },
-            NetworkConfig.MODE_TESTNET: {
-                'identity_registry': ERC8004_V1_IDENTITY,
-                'reputation_registry': ERC8004_V1_REPUTATION,
-                'validation_registry': ERC8004_V1_VALIDATION,
-                'usdc_token': '0x036CbD53842c5426634e7929541eC2318f3dCF7e',  # To be confirmed
+            NetworkConfig.LINEA_SEPOLIA: {
+                'identity_registry': '0x8004aa7C931bCE1233973a0C6A667f73F66282e7',
+                'reputation_registry': '0x8004bd8483b99310df121c46ED8858616b2Bba02',
+                'validation_registry': '0x8004c44d1EFdd699B2A26e781eF7F77c56A9a4EB',
+                'usdc_token': '0x0000000000000000000000000000000000000000',  # TODO: Add Linea USDC
+                'treasury': '0x20E7B2A2c8969725b88Dd3EF3a11Bc3353C83F70'
+            },
+            NetworkConfig.HEDERA_TESTNET: {
+                'identity_registry': '0x4c74ebd72921d537159ed2053f46c12a7d8e5923',
+                'reputation_registry': '0xc565edcba77e3abeade40bfd6cf6bf583b3293e0',
+                'validation_registry': '0x18df085d85c586e9241e0cd121ca422f571c2da6',
+                'usdc_token': '0x0000000000000000000000000000000000000000',  # TODO: Add Hedera USDC
+                'treasury': '0x20E7B2A2c8969725b88Dd3EF3a11Bc3353C83F70'
+            },
+            NetworkConfig.BSC_TESTNET: {
+                'identity_registry': '0xabbd26d86435b35d9c45177725084ee6a2812e40',
+                'reputation_registry': '0xeced1af52a0446275e9e6e4f6f26c99977400a6a',
+                'validation_registry': '0x7866bd057f09a4940fe2ce43320518c8749a921e',
+                'usdc_token': '0x0000000000000000000000000000000000000000',  # TODO: Add BSC USDC
                 'treasury': '0x20E7B2A2c8969725b88Dd3EF3a11Bc3353C83F70'
             },
             NetworkConfig.ZEROG_TESTNET: {
-                'identity_registry': ERC8004_V1_IDENTITY,
-                'reputation_registry': ERC8004_V1_REPUTATION,
-                'validation_registry': ERC8004_V1_VALIDATION,
-                'usdc_token': '0x036CbD53842c5426634e7929541eC2318f3dCF7e',  # To be confirmed
+                'identity_registry': '0x80043ed9cf33a3472768dcd53175bb44e03a1e4a',
+                'reputation_registry': '0x80045d7b72c47bf5ff73737b780cb1a5ba8ee202',
+                'validation_registry': '0x80041728e0aadf1d1427f9be18d52b7f3afefafb',
+                'usdc_token': '0x0000000000000000000000000000000000000000',  # 0G uses native A0GI token
                 'treasury': '0x20E7B2A2c8969725b88Dd3EF3a11Bc3353C83F70'
             }
         }
@@ -120,13 +137,13 @@ class ChaosAgent:
         network_contracts = contract_addresses.get(self.network)
         if not network_contracts:
             raise ConfigurationError(f"No deployed contracts configured for network: {self.network}")
-        
-        self.contract_addresses = ContractAddresses(
+            
+            self.contract_addresses = ContractAddresses(
             identity_registry=network_contracts['identity_registry'],
             reputation_registry=network_contracts['reputation_registry'], 
             validation_registry=network_contracts['validation_registry'],
-            network=self.network
-        )
+                network=self.network
+            )
     
     def _load_contracts(self):
         """Load contract instances with embedded ABIs."""
@@ -169,8 +186,8 @@ class ChaosAgent:
         """
         return [
             # ERC-8004 v1.0 Registration Functions
-            {
-                "inputs": [
+                {
+                    "inputs": [
                     {"name": "tokenURI_", "type": "string"},
                     {
                         "name": "metadata",
@@ -180,13 +197,13 @@ class ChaosAgent:
                             {"name": "value", "type": "bytes"}
                         ]
                     }
-                ],
+                    ],
                 "name": "register",
-                "outputs": [{"name": "agentId", "type": "uint256"}],
-                "stateMutability": "nonpayable",
-                "type": "function"
-            },
-            {
+                    "outputs": [{"name": "agentId", "type": "uint256"}],
+                    "stateMutability": "nonpayable",
+                    "type": "function"
+                },
+                {
                 "inputs": [{"name": "tokenURI_", "type": "string"}],
                 "name": "register",
                 "outputs": [{"name": "agentId", "type": "uint256"}],
@@ -257,8 +274,8 @@ class ChaosAgent:
                 "inputs": [{"name": "tokenId", "type": "uint256"}],
                 "name": "getApproved",
                 "outputs": [{"name": "operator", "type": "address"}],
-                "stateMutability": "view",
-                "type": "function"
+                    "stateMutability": "view",
+                    "type": "function"
             },
             {
                 "inputs": [
@@ -292,19 +309,15 @@ class ChaosAgent:
                 "stateMutability": "view",
                 "type": "function"
             },
-            # View Functions
+            # Additional Functions
             {
-                "inputs": [],
-                "name": "totalAgents",
-                "outputs": [{"name": "count", "type": "uint256"}],
-                "stateMutability": "view",
-                "type": "function"
-            },
-            {
-                "inputs": [{"name": "agentId", "type": "uint256"}],
-                "name": "agentExists",
-                "outputs": [{"name": "exists", "type": "bool"}],
-                "stateMutability": "view",
+                "inputs": [
+                    {"name": "agentId", "type": "uint256"},
+                    {"name": "newUri", "type": "string"}
+                ],
+                "name": "setAgentUri",
+                "outputs": [],
+                "stateMutability": "nonpayable",
                 "type": "function"
             },
             # Events
@@ -359,6 +372,16 @@ class ChaosAgent:
                 ],
                 "name": "ApprovalForAll",
                 "type": "event"
+            },
+            {
+                "anonymous": False,
+                "inputs": [
+                    {"indexed": True, "name": "agentId", "type": "uint256"},
+                    {"indexed": False, "name": "newUri", "type": "string"},
+                    {"indexed": True, "name": "updatedBy", "type": "address"}
+                ],
+                "name": "UriUpdated",
+                "type": "event"
             }
         ]
     
@@ -375,20 +398,20 @@ class ChaosAgent:
         """
         return [
             # Core Functions
-            {
-                "inputs": [
+                {
+                    "inputs": [
                     {"name": "agentId", "type": "uint256"},
                     {"name": "score", "type": "uint8"},
                     {"name": "tag1", "type": "bytes32"},
                     {"name": "tag2", "type": "bytes32"},
-                    {"name": "fileuri", "type": "string"},
-                    {"name": "filehash", "type": "bytes32"},
+                    {"name": "feedbackUri", "type": "string"},
+                    {"name": "feedbackHash", "type": "bytes32"},
                     {"name": "feedbackAuth", "type": "bytes"}
-                ],
+                    ],
                 "name": "giveFeedback",
-                "outputs": [],
-                "stateMutability": "nonpayable",
-                "type": "function"
+                    "outputs": [],
+                    "stateMutability": "nonpayable",
+                    "type": "function"
             },
             {
                 "inputs": [
@@ -497,11 +520,24 @@ class ChaosAgent:
                     {"indexed": False, "name": "score", "type": "uint8"},
                     {"indexed": True, "name": "tag1", "type": "bytes32"},
                     {"indexed": False, "name": "tag2", "type": "bytes32"},
-                    {"indexed": False, "name": "fileuri", "type": "string"},
-                    {"indexed": False, "name": "filehash", "type": "bytes32"}
+                    {"indexed": False, "name": "feedbackUri", "type": "string"},
+                    {"indexed": False, "name": "feedbackHash", "type": "bytes32"}
                 ],
                 "name": "NewFeedback",
                 "type": "event"
+            },
+            {
+                "anonymous": False,
+                "inputs": [
+                    {"indexed": True, "name": "agentId", "type": "uint256"},
+                    {"indexed": True, "name": "clientAddress", "type": "address"},
+                    {"indexed": False, "name": "feedbackIndex", "type": "uint64"},
+                    {"indexed": True, "name": "responder", "type": "address"},
+                    {"indexed": False, "name": "responseUri", "type": "string"},
+                    {"indexed": False, "name": "responseHash", "type": "bytes32"}
+                ],
+                "name": "ResponseAppended",
+                "type": "event"
             }
         ]
     
@@ -518,30 +554,30 @@ class ChaosAgent:
         """
         return [
             # Core Functions
-            {
-                "inputs": [
+                {
+                    "inputs": [
                     {"name": "validatorAddress", "type": "address"},
                     {"name": "agentId", "type": "uint256"},
                     {"name": "requestUri", "type": "string"},
                     {"name": "requestHash", "type": "bytes32"}
-                ],
-                "name": "validationRequest",
-                "outputs": [],
-                "stateMutability": "nonpayable",
-                "type": "function"
-            },
-            {
-                "inputs": [
+                    ],
+                    "name": "validationRequest",
+                    "outputs": [],
+                    "stateMutability": "nonpayable",
+                    "type": "function"
+                },
+                {
+                    "inputs": [
                     {"name": "requestHash", "type": "bytes32"},
-                    {"name": "response", "type": "uint8"},
+                        {"name": "response", "type": "uint8"},
                     {"name": "responseUri", "type": "string"},
                     {"name": "responseHash", "type": "bytes32"},
                     {"name": "tag", "type": "bytes32"}
-                ],
+                    ],
                 "name": "validationResponse",
-                "outputs": [],
-                "stateMutability": "nonpayable",
-                "type": "function"
+                    "outputs": [],
+                    "stateMutability": "nonpayable",
+                    "type": "function"
             },
             # Read Functions
             {
@@ -551,6 +587,7 @@ class ChaosAgent:
                     {"name": "validatorAddress", "type": "address"},
                     {"name": "agentId", "type": "uint256"},
                     {"name": "response", "type": "uint8"},
+                    {"name": "responseHash", "type": "bytes32"},
                     {"name": "tag", "type": "bytes32"},
                     {"name": "lastUpdate", "type": "uint256"}
                 ],
@@ -585,25 +622,6 @@ class ChaosAgent:
                 "stateMutability": "view",
                 "type": "function"
             },
-            {
-                "inputs": [{"name": "requestHash", "type": "bytes32"}],
-                "name": "requestExists",
-                "outputs": [{"name": "exists", "type": "bool"}],
-                "stateMutability": "view",
-                "type": "function"
-            },
-            {
-                "inputs": [{"name": "requestHash", "type": "bytes32"}],
-                "name": "getRequest",
-                "outputs": [
-                    {"name": "validatorAddress", "type": "address"},
-                    {"name": "agentId", "type": "uint256"},
-                    {"name": "requestUri", "type": "string"},
-                    {"name": "timestamp", "type": "uint256"}
-                ],
-                "stateMutability": "view",
-                "type": "function"
-            },
             {
                 "inputs": [],
                 "name": "getIdentityRegistry",
@@ -842,6 +860,66 @@ class ChaosAgent:
         
         return None
     
+    def get_reputation_score(self, agent_id: Optional[int] = None) -> float:
+        """
+        Get reputation score from ERC-8004 Reputation Registry.
+        
+        Fetches all feedback for the agent and computes an average score.
+        
+        Args:
+            agent_id: Agent ID (defaults to self.agent_id)
+        
+        Returns:
+            Average reputation score (0-100)
+        
+        Example:
+            ```python
+            my_reputation = agent.get_reputation_score()
+            print(f"My reputation: {my_reputation}/100")
+            ```
+        """
+        if agent_id is None:
+            agent_id = self.agent_id
+        
+        if not agent_id:
+            rprint("[yellow]⚠️  Agent not registered, cannot fetch reputation[/yellow]")
+            return 0.0
+        
+        try:
+            # Get Reputation Registry address
+            reputation_registry_address = self.registry.functions.getReputationRegistry().call()
+            
+            if reputation_registry_address == "0x0000000000000000000000000000000000000000":
+                rprint("[yellow]⚠️  Reputation Registry not set[/yellow]")
+                return 0.0
+            
+            # Check if it's a deployed contract
+            code = self.w3.eth.get_code(reputation_registry_address)
+            if code == b'' or code == b'0x':
+                rprint("[yellow]⚠️  Reputation Registry not deployed[/yellow]")
+                return 0.0
+            
+            # Load Reputation Registry ABI
+            reputation_abi = self._load_abi("IERC8004Reputation")
+            reputation_registry = self.w3.eth.contract(
+                address=reputation_registry_address,
+                abi=reputation_abi
+            )
+            
+            # Get feedback count for this agent
+            # Note: ERC-8004 Reputation doesn't have a direct "getFeedbackCount" method
+            # We'll need to query events or use a helper method
+            # For now, return a default score (in production, query feedback events)
+            
+            # TODO: Query FeedbackGiven events and compute average
+            # For MVP, return a placeholder
+            rprint(f"[yellow]⚠️  Reputation fetching not fully implemented (agent_id: {agent_id})[/yellow]")
+            return 75.0  # Placeholder score
+            
+        except Exception as e:
+            rprint(f"[yellow]⚠️  Failed to fetch reputation: {e}[/yellow]")
+            return 0.0
+    
     def set_agent_metadata(self, key: str, value: bytes) -> TransactionHash:
         """
         Set on-chain metadata for this agent (ERC-8004 v1.0).
@@ -854,7 +932,7 @@ class ChaosAgent:
         Args:
             key: Metadata key (string)
             value: Metadata value as bytes
-        
+            
         Returns:
             Transaction hash
             
